# -*- coding: UTF-8 -*-
# La ligne ci-dessus permet de spcifier l'encodage, l'UTF-8 est recommand.

# iFree Skype Recorder app module for NVDA

# Importation des modules ncessaires.
import api
import speech
import controlTypes
import eventHandler
import addonHandler
from ui import message
import appModuleHandler
import os

# Pour les traductions.
addonHandler.initTranslation()

# Dclaration d'une variable globale pour la reconnaissance des noms des onglets lors de la navigation tabulaire dans les options.
flag = 0

# Dclaration et initialisation d'une variable globale pour le stockage du paramtre de la parole.
# Elle sera utilise pour couper la parole dans certains contextes.
oldSpeechMode = speech.speechMode

# Quelques lignes pour rcuprer les informations de notre addon.
_addonDir = os.path.join(os.path.dirname(__file__), "..").decode("mbcs") # variable pour stocker le chemin du dossier de l'addon.
_curAddon = addonHandler.Addon(_addonDir) # Objet reprsentant l'addon courant.
_addonSummary = _curAddon.manifest['summary'] # Variable qui stockera le summary de notre addon.

# Cration d'un dictionnaire pour l'tiquetage des boutons du programme, en fonction de leur windowControlID.
# Les boutons "Start recording" et "Pause" dont les windowControlID sont respectivement 1001 et 1002 feront l'objet d'un traitement particulier dans la fonction event_gainFocus
labels = {
	1000: _(u"Close"),
	1011: _(u"History"),
	1012: _(u"Options"),
	1013: _(u"Help"),
	1027: _(u"Stop-most"),
	1059: _(u"Support us"),
	1078: _(u"Stick to Skype"),
	}

def startRecordingOrStop() :
	# Fonction charge de vrifier l'tat de l'enregistrement.
	obj = api.getForegroundObject().firstChild.next.next.next.next.next
	text = obj.name
	if text == u'Ready for recording ...':
		return True
	return False

def pauseOrResume() :
	# Fonction charge de vrifier l'tat de mise en pause/reprise de l'enregistrement.
	obj = api.getForegroundObject().firstChild.next.next.next.next.next.next
	text = obj.name
	if text == u'Recording in progress ...':
		return True
	return False

def goToTabsList(obj):
	# On essaie d'atteindre la fentre parent qui comporte le nom de l'onglet.
	if obj.role == controlTypes.ROLE_BUTTON and obj.name == u"Cancel":
		# Si on est sur le bouton "Cancel", le chemin vers les onglets est un peu diffrent des autres contextes.
		return obj.parent.next.firstChild.firstChild
	# Pour les autres contextes.
	for i in range(3):
		# S'il y a bien un objet parent.
		if obj.parent:
			obj = obj.parent
		else:
			# Pas d'objet parent, on retourne None.
			return None
	# On a bien atteint le troisime objet parent, on vrifie s'il a bien un objet prcdent.
	if obj.previous:
		# On le stocke dans la variable obj.
		obj = obj.previous
	else:
		# Pas d'objet prcdent, on retourne None.
		return None
	# A partir de l, on essaie d'atteindre le second sous enfant, qui devrait comporter la liste des onglets.
	for y in range(2):
		if obj.firstChild:
			# S'il y a bien un objet enfant, on le stocke dans la variable obj.
			obj = obj.firstChild
		else:
			# Pas d'enfant, on retourne None.
			return None
	return obj

def setFocusWithSpeech(obj):
	# Permet d'atteindre l'onglet en question en filtrant ce qui doit tre lu.
	# On coupe la parole.
	speech.speechMode = speech.speechMode_off
	# On met le focus sur l'objet dsir.
	obj.setFocus()
	# On laisse les vnements en attente s'excuter.
	api.processPendingEvents()
	# On redonne la parole.
	speech.speechMode = oldSpeechMode
	# On lit l'objet que l'on vient d'atteindre.
	eventHandler.queueEvent("gainFocus", obj)

def sayCurrentTab(obj):
	#Permet de lire le nom de l'onglet en cours tout en y positionnant le focus.
	# On atteint notre liste d'onglets.
	if goToTabsList(obj):
		# On stocke une variable pour l'objet du premier onglet.
		tbObj1 = goToTabsList(obj)
		# On stocke une variable pour l'objet du second onglet.
		tbObj2 = tbObj1.next
		# On stocke une variable pour l'objet du troisime onglet.
		tbObj3 = tbObj2.next
		# On stocke une variable pour l'objet du quatrime onglet.
		tbObj4 = tbObj3.next
	# Si on est sur le bouton Annuler.
	if obj.role == controlTypes.ROLE_BUTTON and obj.name == u"Cancel":
		# Le chemin est un peu diffrent des autres contextes.
		obj = obj.parent.next.next.firstChild.firstChild
		# Pour l'onglet "Call Record" en particulier.
		if obj.windowControlID == -1:
			obj = obj.next
	# On cre un dictionnaire pour stocker les objets  atteindre selon chaque controlID.
	tabs={
	1021: tbObj1,
	1032: tbObj2,
	1033: tbObj2,
	1042: tbObj3,
	1043: tbObj3,
	1044: tbObj3,
	1058: tbObj4
	}
	# On dplace le focus vers l'onglet en question en filtrant ce qui sera dit.
	setFocusWithSpeech(tabs[obj.windowControlID])

def inOptionsDialog(obj):
	# Fonction pour vrifier si l'on est bien dans la bote de dialogue des options de iFree Skype Recorder.
	for i in range(4):
		# Si l'on peut remonter au quatrime parent.
		if obj.parent:
			# On stocke la valeur de l'objet aprent dans la variable obj.
			obj = obj.parent
		else:
			# On ne peut pas remonter  l'objet parent, on retourne False.
			return False
	# On est bien sur le quatrime objet parent, on vrifie s'il a un nom et si ce nom est u"Options".
	if obj.name and obj.name == u"Options":
		# La condition est remplie, on retourne True.
		return True
	# La condition n'est pas remplie, on retourne False.
	return False

def getCurTabName(obj):
	# Fonction permettant de rcuprer le nom de l'onglet courant.
	for i in range(3):
		# Si on peut remonter au troisime parent.
		if obj.parent:
			# On stocke la valeur de l'objet parent dans la variable obj.
			obj = obj.parent
		else:
			# Pas de parent, on retourne None.
			return None
	# L'objet en question a bien t trouv, on retourne son name.
	return obj.name	

class AppModule(appModuleHandler.AppModule):
	scriptCategory = _addonSummary

	def event_gainFocus(self, obj, nextHandler):
		# Utilisation de la variable globale flag.
		global flag
		# Traitement de la variabilit du nom du bouton d'enregistrement.
		if obj.windowControlID == 1001:
			obj.name = _(u"Start recording") if startRecordingOrStop() else _(u"Stop recording")
		# Traitement de la variabilit du nom du bouton de pause.
		if obj.windowControlID == 1002:
			obj.name = _(u"Pause") if pauseOrResume() else _(u"Resume")
		# Traitement de la lecture des noms des onglets dans la bote de dialogue des options.
		if obj.role == controlTypes.ROLE_BUTTON and obj.name == u"Apply":
			flag = 1
		if obj.role == controlTypes.ROLE_BUTTON and obj.name == u"Cancel":
			if flag == 0 or flag == 2:
				return sayCurrentTab(obj)
			flag = 0
		if (obj.role == controlTypes.ROLE_CHECKBOX and obj.name == u'Enable automatically chat reply') or obj.windowControlID in [1021, 1032, 1033, 1042, 1043, 1044]:
			if flag == 0:
				return sayCurrentTab(obj)
			flag = 2
		if obj.role == controlTypes.ROLE_TAB:
			flag = 1
		nextHandler()

	def event_NVDAObject_init(self, obj):
		# Traitement de l'tiquetage de tous les boutons dont les windowControlID sont rpertoris dans le dictionnaire label.
		if obj.name != u'Browse' and obj.role == controlTypes.ROLE_BUTTON and obj.windowControlID in [1000, 1011, 1012, 1013, 1027, 1059, 1078]:
			obj.name = labels[obj.windowControlID]

	def script_nextTab(self, gesture):
		# Permet d'aller  l'onglet suivant dans les onglets des options de iFree Skype Recorder.
		# Ces onglets apparaissent juste une tabulation aprs le bouton "Cancel".
		focus = obj = api.getFocusObject()
		if not inOptionsDialog(focus):
			# Nous ne sommes pas dans les options, on excute le raccourci-clavier du script.
			gesture.send()
			# On quitte le script.
			return
		if obj.role == controlTypes.ROLE_TAB:
			# Si l'on est dans les onglets.
			# On quitte la parole pour une meilleure lecture de l'onglet courant.
			speech.speechMode = speech.speechMode_off
			# Si l'objet suivant est disponible.
			if obj.next:
				obj = obj.next
				# On y dplace le focus.
				obj.setFocus()
			else:
				# On est sur le dernier onglet, on revient au tout premier.
				obj = obj.previous.previous.previous
				# On y place le focus.
				obj.setFocus()
			# On laisse les evnements en attente s'excuter.
			api.processPendingEvents()
			# On redonne la parole.
			speech.speechMode = oldSpeechMode
			# On lit l'objet que l'on vient d'atteindre, c'est  dire le nom de l'onglet courant.
			eventHandler.queueEvent("gainFocus", obj)
			# On quitte le script.
			return
		# Si la liste des onglets est bien disponible.
		if goToTabsList(focus):
			# On stocke la valeur du premir objet onglet dans obj.
			obj = goToTabsList(focus)
		else:
			# Pas d'onglets, on quitte le script.
			return
		# On est bien sur nos onglets, on stocke le premier objet dans une variable.
		firstObj = obj
		# On l'insert dans une variable de type liste, intitule tabsList.
		tabsList = [firstObj]
		while obj.next and obj.next.name:
			# Tant qu'il y a un objet suivant et qu'il a un nom, on l'ajoute dans notre liste tabsList.
			obj = obj.next
			tabsList.append(obj)
		# On cre une variable goto de type liste, qui va stocker comme lment de liste, l'objet reprsentant l'onglet en cours dans la liste des onglets.
		goto = [x for x in tabsList if getCurTabName(focus) in x.name or (u"Advanced Settings", u"MP3 Quality") == (getCurTabName(focus), x.name)]
		# On lui affecte la valeur de son seul et unique lment, qui n'est autre que l'objet trouv.
		goto = goto[0]
		# S'il existe bien un onglet suivant.
		if goto.next:
			# On le stocke dans la variable goto.
			goto = goto.next
		else:
			# Il n'y en a pas, on stocke le premier objet ou onglet.
			goto = tabsList[0]
		# On quitte la parole, pour une meilleure lecture de l'onglet.
		speech.speechMode = speech.speechMode_off
		# On dplace le focus vers l'objet en question.
		goto.setFocus()
		# On laisse les vnements en attente s'excuter.
		api.processPendingEvents()
		# On redonne la parole.
		speech.speechMode = oldSpeechMode
		# On lit l'objet que l'on vient d'atteindre.
		eventHandler.queueEvent("gainFocus", goto)
	# Documentation du script pour l'aide-clavier.
	script_nextTab.__doc__ = _(u"Allows you to go to the next tab in the Options dialog. These tabs are availables just a tab after the Cancel button.")

	def script_previousTab(self, gesture):
		# Permet d'aller  l'onglet prcdent dans les onglets des options de iFree Skype Recorder.
		# Ces onglets apparaissent juste une tabulation aprs le bouton "Cancel".
		focus = obj = api.getFocusObject()
		if not inOptionsDialog(focus):
			# Nous ne sommes pas dans les options, on excute le raccourci-clavier du script.
			gesture.send()
			# On quitte le script.
			return
		if obj.role == controlTypes.ROLE_TAB:
			# Si l'on est dans les onglets.
			# On quitte la parole pour une meilleure lecture de l'onglet en cours.
			speech.speechMode = speech.speechMode_off
			# Si l'objet prcdent est disponible.
			if obj.previous:
				obj = obj.previous
				# On y dplace le focus.
				obj.setFocus()
			else:
				# On est sur le premier onglet, on revient au tout dernier.
				obj = obj.next.next.next
				# On y place le focus.
				obj.setFocus()
			# On laisse les evnements en attente s'excuter.
			api.processPendingEvents()
			# On redonne la parole.
			speech.speechMode = oldSpeechMode
			# On lit l'objet que l'on vient d'atteindre, c'est  dire le nom de l'onglet courant.
			eventHandler.queueEvent("gainFocus", obj)
			# On quitte le script.
			return
		# Si la liste des onglets est bien disponible.
		if goToTabsList(focus):
			# On stocke la valeur du premir objet onglet dans obj.
			obj = goToTabsList(focus)
		else:
			# Pas d'onglets, on quitte le script.
			return
		# On est bien sur nos onglets, on stocke le premier objet dans une variable.
		firstObj = obj
		# On l'insert dans une variable de type liste, intitul tabsList.
		tabsList = [firstObj]
		while obj.next and obj.next.name:
			# Tant qu'il y a un objet suivant et qu'il a un nom, on l'ajoute dans notre liste tabsList.
			obj = obj.next
			tabsList.append(obj)
		# On cre une variable goto de type liste, qui va stocker comme lment de liste, l'objet reprsentant l'onglet en cours dans la liste des onglets.
		goto = [x for x in tabsList if getCurTabName(focus) in x.name or (u"Advanced Settings", u"MP3 Quality") == (getCurTabName(focus), x.name)]
		# On lui affecte la valeur de son seul et unique lment, qui n'est autre que l'objet trouv.
		goto = goto[0]
		# S'il existe bien un onglet prcdent.
		if goto.previous:
			# On le stocke dans la variable goto.
			goto = goto.previous
		else:
			# Il n'y en a pas, on stocke le dernier objet ou onglet.
			goto = tabsList[-1]
		# On quitte la parole, pour une meilleure lecture de l'onglet.
		speech.speechMode = speech.speechMode_off
		# On dplace le focus vers l'objet en question.
		goto.setFocus()
		# On laisse les vnements en attente s'excuter.
		api.processPendingEvents()
		# On redonne la parole.
		speech.speechMode = oldSpeechMode
		# On lit l'objet que l'on vient d'atteindre.
		eventHandler.queueEvent("gainFocus", goto)
	# Documentation du script pour l'aide-clavier.
	script_previousTab.__doc__ = _(u"Allows you to go to the previous tab in the Options dialog. These tabs are availables just a tab after the Cancel button.")

	# Dictionnaire des raccourcis-clavier.
	__gestures={
		"kb:control+tab":"nextTab",
		"kb:control+shift+tab":"previousTab"
	}